@@ -147,17 +147,21 @@ escape_pathname (const char *dir)
   while (*dir == '/')
     dir++;
 
-  /* Underscores are doubled, count them */
+  /* count characters that need to be escaped. */
   count = 0;
   p = dir;
   while (*p)
     {
       if (*p == '_')
         count++;
+      if (*p == '/')
+        count++;
+      if (*p == '%')
+        count++;
       p++;
     }
   
-  res = g_malloc (strlen (dir) + count + 1);
+  res = g_malloc (strlen (dir) + count*2 + 1);
   
   p = dir;
   d = res;
@@ -166,18 +170,27 @@ escape_pathname (const char *dir)
       c = *p++;
       if (c == '_')
         {
-          *d++ = '_';
-          *d++ = '_';
+          *d++ = '%';
+          *d++ = '5';
+          *d++ = 'f';
         }
       else if (c == '/')
         {
-          *d++ = '_';
+          *d++ = '%';
+          *d++ = '2';
+          *d++ = 'f';
           
           /* Skip consecutive slashes, they are unnecessary,
              and break our escaping */
           while (*p == '/')
             p++;
         }
+      else if (c == '%')
+        {
+          *d++ = '%';
+          *d++ = '2';
+          *d++ = '5';
+        }
       else
         *d++ = c;
     }
@@ -215,20 +228,22 @@ unescape_pathname (const char *escaped_dir, int len)
   while (p < end)
     {
       c = *p++;
-      if (c == '_')
+      if (c == '%' && p < (end-1))
         {
-          if (p == end)
-            *d++ = '_';
-          else
+          if (*(p) == '2' && *(p+1) == 'f')
             {
-              c = *p;
-              if (c == '_')
-                {
-                  p++;
-                  *d++ = '_';
-                }
-              else
-                *d++ = '/';
+              *d++ = '/';
+              p+=2;
+            }
+          else if (*(p) == '2' && *(p+1) == '5')
+            {
+              *d++ = '%';
+              p+=2;
+            }
+          else if (*(p) == '5' && *(p+1) == 'f')
+            {
+              *d++ = '_';
+              p+=2;
             }
         }
       else
@@ -516,12 +531,32 @@ list_trash_dirs (void)
   GList *topdirs_info;
   struct stat statbuf;
   gboolean has_trash_files;
+  int stat_result;
 
   dirs = NULL;
   has_trash_files = FALSE;
   
   home_trash = g_build_filename (g_get_user_data_dir (), "Trash", NULL);
-  if (lstat (home_trash, &statbuf) == 0 &&
+
+  stat_result = g_lstat (home_trash, &statbuf);
+
+  /* If the home trash directory doesn't exist at this point, we must create
+   * it in order to monitor it. */
+  if (stat_result != 0)
+    {
+      gchar *home_trash_files = g_build_filename (home_trash, "files", NULL);
+      gchar *home_trash_info  = g_build_filename (home_trash, "info", NULL);
+
+      g_mkdir_with_parents (home_trash_files, 0700);
+      g_mkdir_with_parents (home_trash_info, 0700);
+
+      g_free (home_trash_files);
+      g_free (home_trash_info);
+
+      stat_result = g_lstat (home_trash, &statbuf);
+    }
+
+  if (stat_result == 0 &&
       S_ISDIR (statbuf.st_mode))
     {
       dirs = g_list_prepend (dirs, home_trash);
@@ -537,8 +572,11 @@ list_trash_dirs (void)
     {
       mount = l->data;
       
-      topdir = g_unix_mount_get_mount_path (mount);
-      topdirs = g_list_prepend (topdirs, g_strdup (topdir));
+      if (!g_unix_mount_is_system_internal (mount) )
+        {
+          topdir = g_unix_mount_get_mount_path (mount);
+          topdirs = g_list_prepend (topdirs, g_strdup (topdir));
+        }
       
       g_unix_mount_free (mount);
     }
