@@ -31,7 +31,7 @@
  *    - neglible memory + cpu overhead
  *      - will need to construct them at some point
  *    - we can actually implement get_mount_for_mount_path()
- *      correctly
+ *      correctly even when no volume monitor is constructed
  *
  *  - implement support for GMountOperation
  *    - not implemented in the HAL volume monitor and that's all
@@ -264,42 +264,45 @@ static GMount *
 get_mount_for_mount_path (const char *mount_path,
                           GCancellable *cancellable)
 {
-  GType type;
   GMount *mount;
-  GNativeVolumeMonitorClass *klass;
+  GProxyVolumeMonitor *volume_monitor;
+  GProxyVolumeMonitorClass *klass;
+  GHashTableIter vm_hash_iter;
+  GHashTableIter vol_hash_iter;
+  GProxyMount *candidate_mount;
 
   /* This static method on GNativeVolumeMonitor is a *complete* pain
    * in the ass to deal with; we need to rework gio so it's deprecated
    * and thus never will get called.
    *
-   * For now we just implement as badly as the GUnixVolumeMonitor
-   * shipped with gio; e.g. we just return *some* object that
-   * implements GMount and we don't worry about setting the
-   * corresponding volume or ensuring it's the same reference as what
-   * one would normally get.
+   * TODO: we don't handle the case when there's no volume monitor ever
+   * constructed. See TODO at the top of this file on how to handle that.
    */
 
   mount = NULL;
-  klass = NULL;
+  G_LOCK (proxy_vm);
 
-  /* You can't hide types from me
-   *
-   * (read: _g_unix_volume_monitor_get_type() isn't exported)
-   */
-  type = g_type_from_name ("GUnixVolumeMonitor");
-  if (type == 0)
-    goto out;
+  /* First find the native volume monitor if one exists */
+  g_hash_table_iter_init (&vm_hash_iter, the_volume_monitors);
+  while (g_hash_table_iter_next (&vm_hash_iter, NULL, (gpointer) &volume_monitor)) {
+    klass = G_PROXY_VOLUME_MONITOR_CLASS (G_OBJECT_GET_CLASS (volume_monitor));
 
-  klass = G_NATIVE_VOLUME_MONITOR_CLASS (g_type_class_ref (type));
-  if (klass == NULL)
-    goto out;
-
-  if (klass->get_mount_for_mount_path != NULL)
-    mount = klass->get_mount_for_mount_path (mount_path, cancellable);
+    if (klass->is_native) {
+      /* The see if we've got a mount */
+      g_hash_table_iter_init (&vol_hash_iter, volume_monitor->mounts);
+      while (g_hash_table_iter_next (&vol_hash_iter, NULL, (gpointer) &candidate_mount)) {
+        if (g_proxy_mount_has_mount_path (candidate_mount, mount_path))
+          {
+            mount = g_object_ref (candidate_mount);
+            goto out;
+          }
+      }
+      goto out;
+    }
+  }
 
  out:
-  if (klass != NULL)
-    g_type_class_unref (klass);
+  G_UNLOCK (proxy_vm);
   return mount;
 }
 
@@ -418,13 +421,17 @@ filter_function (DBusConnection *connection, DBusMessage *message, void *user_da
   GProxyVolumeMonitor *monitor = G_PROXY_VOLUME_MONITOR (user_data);
   DBusMessageIter iter;
   const char *id;
+  const char *the_dbus_name;
   const char *member;
   GProxyDrive *drive;
   GProxyVolume *volume;
   GProxyMount *mount;
+  GProxyVolumeMonitorClass *klass;
 
   G_LOCK (proxy_vm);
 
+  klass = G_PROXY_VOLUME_MONITOR_CLASS (G_OBJECT_GET_CLASS (monitor));
+
   member = dbus_message_get_member (message);
 
   if (dbus_message_is_signal (message, "org.gtk.Private.RemoteVolumeMonitor", "DriveChanged") ||
@@ -433,9 +440,14 @@ filter_function (DBusConnection *connection, DBusMessage *message, void *user_da
       dbus_message_is_signal (message, "org.gtk.Private.RemoteVolumeMonitor", "DriveEjectButton")) {
 
     dbus_message_iter_init (message, &iter);
+    dbus_message_iter_get_basic (&iter, &the_dbus_name);
+    dbus_message_iter_next (&iter);
     dbus_message_iter_get_basic (&iter, &id);
     dbus_message_iter_next (&iter);
 
+    if (strcmp (the_dbus_name, klass->dbus_name) != 0)
+      goto not_for_us;
+
     if (strcmp (member, "DriveChanged") == 0)
       {
         drive = g_hash_table_lookup (monitor->drives, id);
@@ -484,9 +496,14 @@ filter_function (DBusConnection *connection, DBusMessage *message, void *user_da
              dbus_message_is_signal (message, "org.gtk.Private.RemoteVolumeMonitor", "VolumeRemoved")) {
 
     dbus_message_iter_init (message, &iter);
+    dbus_message_iter_get_basic (&iter, &the_dbus_name);
+    dbus_message_iter_next (&iter);
     dbus_message_iter_get_basic (&iter, &id);
     dbus_message_iter_next (&iter);
 
+    if (strcmp (the_dbus_name, klass->dbus_name) != 0)
+      goto not_for_us;
+
     if (strcmp (member, "VolumeChanged") == 0)
       {
         volume = g_hash_table_lookup (monitor->volumes, id);
@@ -527,9 +544,14 @@ filter_function (DBusConnection *connection, DBusMessage *message, void *user_da
              dbus_message_is_signal (message, "org.gtk.Private.RemoteVolumeMonitor", "MountRemoved")) {
 
     dbus_message_iter_init (message, &iter);
+    dbus_message_iter_get_basic (&iter, &the_dbus_name);
+    dbus_message_iter_next (&iter);
     dbus_message_iter_get_basic (&iter, &id);
     dbus_message_iter_next (&iter);
 
+    if (strcmp (the_dbus_name, klass->dbus_name) != 0)
+      goto not_for_us;
+
     if (strcmp (member, "MountChanged") == 0)
       {
         mount = g_hash_table_lookup (monitor->mounts, id);
@@ -575,6 +597,7 @@ filter_function (DBusConnection *connection, DBusMessage *message, void *user_da
 
   }
 
+ not_for_us:
   G_UNLOCK (proxy_vm);
 
   return DBUS_HANDLER_RESULT_NOT_YET_HANDLED;
@@ -591,10 +614,27 @@ g_proxy_volume_monitor_class_finalize (GProxyVolumeMonitorClass *klass)
   g_free (klass->dbus_name);
 }
 
+typedef struct {
+  char *dbus_name;
+  gboolean is_native;
+} ProxyClassData;
+
+static ProxyClassData *
+proxy_class_data_new (const char *dbus_name, gboolean is_native)
+{
+  ProxyClassData *data;
+  data = g_new0 (ProxyClassData, 1);
+  data->dbus_name = g_strdup (dbus_name);
+  data->is_native = is_native;
+  return data;
+}
+
 static void
 g_proxy_volume_monitor_class_intern_init_pre (GProxyVolumeMonitorClass *klass, gconstpointer class_data)
 {
-  klass->dbus_name = g_strdup ((char *) class_data);
+  ProxyClassData *data = (ProxyClassData *) class_data;
+  klass->dbus_name = g_strdup (data->dbus_name);
+  klass->is_native = data->is_native;
   g_proxy_volume_monitor_class_intern_init (klass);
 }
 
@@ -841,36 +881,19 @@ _get_identifiers (DBusMessageIter *iter)
   return hash_table;
 }
 
-GIcon *
-_g_icon_new_from_serialized_data (const char *gicon_data)
+static GIcon *
+_g_icon_new_from_tokens (char **tokens, int num_tokens)
 {
-  char **tokens;
   GIcon *icon;
-  guint num_tokens;
-
-  g_return_val_if_fail (gicon_data != NULL, NULL);
-
+  
   icon = NULL;
-
-  tokens = g_strsplit (gicon_data, " ", 0);
-  num_tokens = g_strv_length (tokens);
-
-  if (num_tokens < 2)
-    {
-      g_warning ("malformed GIcon data \"%s\"", gicon_data);
-      goto out;
-    }
-
   if (strcmp (tokens[0], "GFileIcon") == 0)
     {
       GFile *file;
       char *unescaped_uri;
 
       if (num_tokens != 2)
-        {
-          g_warning ("malformed GFileIcon gicon_data \"%s\"", gicon_data);
-          goto out;
-        }
+        goto out;
 
       unescaped_uri = g_uri_unescape_string (tokens[1], NULL);
       file = g_file_new_for_uri (unescaped_uri);
@@ -882,7 +905,6 @@ _g_icon_new_from_serialized_data (const char *gicon_data)
     {
       int n;
 
-      icon = NULL;
       for (n = 1; n < num_tokens; n++)
         {
           char *unescaped_name;
@@ -895,12 +917,79 @@ _g_icon_new_from_serialized_data (const char *gicon_data)
           g_free (unescaped_name);
         }
     }
-  else
+  else if (strcmp (tokens[0], "GEmblemedIcon") == 0)
     {
-      g_warning ("cannot parse gicon_data \"%s\"; please add support", gicon_data);
+       int n, m, i;
+       GIcon *base, *e_icon;
+       GEmblem *emblem;
+       GList *emblems, *e;
+       GEmblemOrigin origin;
+       char **t = tokens;
+
+       t++;
+       n = atoi (*t);
+       t++;
+       if (t - tokens >= num_tokens) 
+         goto out;
+       base = _g_icon_new_from_tokens (t, n);
+       if (base == NULL)
+         goto out;
+
+       t = t + n;
+       m = atoi (*t);
+       t++;
+       emblems = NULL;
+       for (i = 0; i < m; i++)
+         {
+           origin = atoi (*t);
+           t++;
+           n = atoi (*t);
+           t++;
+           if (t - tokens >= num_tokens) 
+             goto cleanup;
+           e_icon = _g_icon_new_from_tokens (t, n);
+           t += n;
+           if (e_icon == NULL)
+             goto cleanup;
+           emblem = g_emblem_new_with_origin (e_icon, origin);
+           emblems = g_list_append (emblems, emblem);
+         }
+       icon = g_emblemed_icon_new (base, (GEmblem*)emblems->data);
+       for (e = emblems->next; e; e = e->next)
+         g_emblemed_icon_add_emblem (G_EMBLEMED_ICON (icon), (GEmblem*)e->data);
+
+ cleanup:
+       g_object_unref (base);
+       g_list_foreach (emblems, (GFunc)g_object_unref, NULL);
+       g_list_free (emblems);
     }
 
  out:
+   return icon;
+}
+
+GIcon *
+_g_icon_new_from_serialized_data (const char *gicon_data)
+{
+  char **tokens;
+  GIcon *icon;
+  gint num_tokens;
+
+  g_return_val_if_fail (gicon_data != NULL, NULL);
+
+  icon = NULL;
+
+  tokens = g_strsplit (gicon_data, " ", 0);
+
+  if (g_strv_length (tokens) >= 3)
+    {
+      num_tokens = atoi (tokens[0]);
+      icon = _g_icon_new_from_tokens (tokens + 1, num_tokens);
+    }
+
+  if (icon == NULL)
+    g_warning ("malformed GIcon data \"%s\"", gicon_data);
+
   g_strfreev (tokens);
   return icon;
 }
@@ -932,7 +1021,7 @@ register_volume_monitor (GTypeModule *type_module,
     (GBaseFinalizeFunc) NULL,
     (GClassInitFunc) g_proxy_volume_monitor_class_intern_init_pre,
     (GClassFinalizeFunc) g_proxy_volume_monitor_class_finalize,
-    (gconstpointer) g_strdup (dbus_name),  /* class_data (leaked!) */
+    (gconstpointer) proxy_class_data_new (dbus_name, is_native),  /* class_data (leaked!) */
     sizeof (GProxyVolumeMonitor),
     0,      /* n_preallocs */
     (GInstanceInitFunc) g_proxy_volume_monitor_init,
